@@ -329,15 +329,17 @@ cperf_throughput_test_runner(void *test_ctx)
 	struct cperf_benchmark_ctx *ctx = test_ctx;
 	struct comp_test_data *test_data = ctx->ver.options;
 	uint32_t lcore = rte_lcore_id();
-	static rte_atomic16_t display_once = RTE_ATOMIC16_INIT(0);
+	static uint16_t display_once;
 	int i, ret = EXIT_SUCCESS;
 
 	ctx->ver.mem.lcore_id = lcore;
 
+	uint16_t exp = 0;
 	/*
 	 * printing information about current compression thread
 	 */
-	if (rte_atomic16_test_and_set(&ctx->ver.mem.print_info_once))
+	if (__atomic_compare_exchange_n(&ctx->ver.mem.print_info_once, &exp,
+				1, 0, __ATOMIC_RELAXED, __ATOMIC_RELAXED))
 		printf("    lcore: %u,"
 				" driver name: %s,"
 				" device name: %s,"
@@ -355,55 +357,45 @@ cperf_throughput_test_runner(void *test_ctx)
 	 * First the verification part is needed
 	 */
 	if (cperf_verify_test_runner(&ctx->ver)) {
-		ret = EXIT_FAILURE;
+		ret =  EXIT_FAILURE;
 		goto end;
 	}
 
-	if (test_data->test_op & COMPRESS) {
-		/*
-		 * Run the test twice, discarding the first performance
-		 * results, before the cache is warmed up
-		 */
-		for (i = 0; i < 2; i++) {
-			if (main_loop(ctx, RTE_COMP_COMPRESS) < 0) {
-				ret = EXIT_FAILURE;
-				goto end;
-			}
+	/*
+	 * Run the tests twice, discarding the first performance
+	 * results, before the cache is warmed up
+	 */
+	for (i = 0; i < 2; i++) {
+		if (main_loop(ctx, RTE_COMP_COMPRESS) < 0) {
+			ret = EXIT_FAILURE;
+			goto end;
 		}
+	}
 
-		ctx->comp_tsc_byte =
+	for (i = 0; i < 2; i++) {
+		if (main_loop(ctx, RTE_COMP_DECOMPRESS) < 0) {
+			ret = EXIT_FAILURE;
+			goto end;
+		}
+	}
+
+	ctx->comp_tsc_byte =
 			(double)(ctx->comp_tsc_duration[test_data->level]) /
-						       test_data->input_data_sz;
-		ctx->comp_gbps = rte_get_tsc_hz() / ctx->comp_tsc_byte * 8 /
-								     1000000000;
-	} else {
-		ctx->comp_tsc_byte = 0;
-		ctx->comp_gbps = 0;
-	}
+					test_data->input_data_sz;
 
-	if (test_data->test_op & DECOMPRESS) {
-		/*
-		 * Run the test twice, discarding the first performance
-		 * results, before the cache is warmed up
-		 */
-		for (i = 0; i < 2; i++) {
-			if (main_loop(ctx, RTE_COMP_DECOMPRESS) < 0) {
-				ret = EXIT_FAILURE;
-				goto end;
-			}
-		}
-
-		ctx->decomp_tsc_byte =
+	ctx->decomp_tsc_byte =
 			(double)(ctx->decomp_tsc_duration[test_data->level]) /
-						       test_data->input_data_sz;
-		ctx->decomp_gbps = rte_get_tsc_hz() / ctx->decomp_tsc_byte * 8 /
-								     1000000000;
-	} else {
-		ctx->decomp_tsc_byte = 0;
-		ctx->decomp_gbps = 0;
-	}
+					test_data->input_data_sz;
 
-	if (rte_atomic16_test_and_set(&display_once)) {
+	ctx->comp_gbps = rte_get_tsc_hz() / ctx->comp_tsc_byte * 8 /
+			1000000000;
+
+	ctx->decomp_gbps = rte_get_tsc_hz() / ctx->decomp_tsc_byte * 8 /
+			1000000000;
+
+	exp = 0;
+	if (__atomic_compare_exchange_n(&display_once, &exp, 1, 0,
+			__ATOMIC_RELAXED, __ATOMIC_RELAXED)) {
 		printf("\n%12s%6s%12s%17s%15s%16s\n",
 			"lcore id", "Level", "Comp size", "Comp ratio [%]",
 			"Comp [Gbps]", "Decomp [Gbps]");
